import { useRef, useEffect, useCallback } from 'react';
import '../utils/useSealedState.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-94b779f5.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../system/useToken.js';
import '../system/useOptions.js';
import '../system/useProps.js';
import '../chunk-2f16e450.js';
import '../chunk-56719312.js';
import { unstable_createHook } from '../utils/createHook.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import { a as mergeRefs } from '../chunk-b94938c6.js';
import { a as useAllCallbacks } from '../chunk-a259d6a5.js';
import { useTabbable } from '../Tabbable/Tabbable.js';
import { a as removeIndexFromArray } from '../chunk-29332083.js';
import { a as warning } from '../chunk-79998200.js';
import { useCheckboxState } from './CheckboxState.js';

var defaultClickKeys = [" "];
var useCheckbox = unstable_createHook({
  name: "Checkbox",
  compose: useTabbable,
  useState: useCheckboxState,
  keys: ["value", "checked"],
  useProps: function useProps(options, _ref) {
    var htmlRef = _ref.ref,
        htmlOnChange = _ref.onChange,
        htmlOnClick = _ref.onClick,
        htmlProps = _objectWithoutPropertiesLoose(_ref, ["ref", "onChange", "onClick"]);

    var ref = useRef(null);
    var isBoolean = typeof options.value === "undefined";
    var checked = typeof options.checked !== "undefined" ? options.checked : isBoolean ? Boolean(options.state) : (options.state || []).indexOf(options.value) !== -1;
    useEffect(function () {
      if (!ref.current) {
        process.env.NODE_ENV !== "production" ? warning(options.state === "indeterminate", "Checkbox", "Can't set indeterminate state because `ref` wasn't passed to component.", "See https://reakit.io/docs/checkbox") : void 0;
        return;
      }

      if (options.state === "indeterminate") {
        ref.current.indeterminate = true;
      } else if (ref.current.indeterminate) {
        ref.current.indeterminate = false;
      }
    }, [options.state]);
    var onChange = useCallback(function (event) {
      if (options.disabled) return;

      if (htmlOnChange) {
        htmlOnChange(event);
      }

      if (!options.setState) return;

      if (isBoolean) {
        options.setState(!checked);
      } else {
        var array = Array.isArray(options.state) ? options.state : [];
        var index = array.indexOf(options.value);

        if (index === -1) {
          options.setState([].concat(array, [options.value]));
        } else {
          options.setState(removeIndexFromArray(array, index));
        }
      }
    }, [htmlOnChange, options.disabled, options.setState, options.state, options.value]);
    var onClick = useCallback(function (event) {
      if (event.target instanceof HTMLInputElement) return;
      onChange(event);
    }, [onChange]);
    return _objectSpread({
      ref: mergeRefs(ref, htmlRef),
      checked: checked,
      "aria-checked": options.state === "indeterminate" ? "mixed" : checked,
      value: options.value,
      role: "checkbox",
      type: "checkbox",
      onChange: onChange,
      onClick: useAllCallbacks(onClick, htmlOnClick)
    }, htmlProps);
  },
  useCompose: function useCompose(options, htmlProps) {
    return useTabbable(_objectSpread({}, options, {
      unstable_clickKeys: defaultClickKeys
    }), htmlProps);
  }
});
var Checkbox = unstable_createComponent({
  as: "input",
  useHook: useCheckbox
});

export { Checkbox, useCheckbox };
