import { useReducer, useCallback, useEffect } from 'react';
import { unstable_useSealedState } from '../utils/useSealedState.js';
import { unstable_useId } from '../utils/useId.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-94b779f5.js';
import '../chunk-2f16e450.js';
import { a as isObject } from '../chunk-56719312.js';
import { a as useUpdateEffect } from '../chunk-b4d83ecc.js';
import { unstable_setAllIn } from './utils/setAllIn.js';
import { unstable_getIn } from './utils/getIn.js';
import { unstable_setIn } from './utils/setIn.js';

function isPromise(arg) {
  return Boolean(arg && arg.then);
}

function isEmpty(arg) {
  if (Array.isArray(arg)) return !arg.length;
  if (isObject(arg)) return !Object.keys(arg).length;
  if (arg == null) return true;
  if (arg === "") return true;
  return false;
}

function filterAllEmpty(object) {
  if (Array.isArray(object)) {
    return object.filter(function (value) {
      if (isObject(value)) {
        return filterAllEmpty(value);
      }

      return true;
    });
  }

  var result = {};
  var keys = Object.keys(object);

  for (var _i = 0, _keys = keys; _i < _keys.length; _i++) {
    var key = _keys[_i];
    var k = key;
    var value = object[k];
    result[k] = isObject(value) ? filterAllEmpty(value) : object[k];
  }

  return result;
}

function hasMessages(errors) {
  return isObject(errors) && !isEmpty(errors);
}

function getMessages(stateMessages, actionMessages) {
  return !isEmpty(actionMessages) ? actionMessages : isEmpty(stateMessages) ? stateMessages : {};
}

function reducer(state, action) {
  switch (action.type) {
    case "reset":
      {
        return _objectSpread({}, state, {
          values: state.initialValues,
          touched: {},
          errors: {},
          messages: {},
          valid: true,
          validating: false,
          submitting: false,
          submitFailed: 0,
          submitSucceed: 0
        });
      }

    case "startValidate":
      {
        return _objectSpread({}, state, {
          validating: true
        });
      }

    case "endValidate":
      {
        return _objectSpread({}, state, {
          validating: false,
          errors: getMessages(state.errors, action.errors),
          messages: getMessages(state.messages, action.messages),
          valid: !hasMessages(action.errors)
        });
      }

    case "startSubmit":
      {
        return _objectSpread({}, state, {
          // @ts-ignore TS bug
          touched: unstable_setAllIn(state.values, true),
          submitting: true
        });
      }

    case "endSubmit":
      {
        var valid = !hasMessages(action.errors);
        return _objectSpread({}, state, {
          valid: valid,
          submitting: false,
          errors: getMessages(state.errors, action.errors),
          messages: getMessages(state.messages, action.messages),
          submitSucceed: valid ? state.submitSucceed + 1 : state.submitSucceed,
          submitFailed: valid ? state.submitFailed : state.submitFailed + 1
        });
      }

    case "update":
      {
        var _name = action.name,
            _value = action.value;
        var nextValue = typeof _value === "function" ? _value(unstable_getIn(state.values, _name)) : _value;
        return _objectSpread({}, state, {
          values: unstable_setIn(state.values, _name, nextValue != null ? nextValue : "")
        });
      }

    case "blur":
      {
        return _objectSpread({}, state, {
          touched: unstable_setIn(state.touched, action.name, true)
        });
      }

    case "push":
      {
        var array = unstable_getIn(state.values, action.name, []);
        return _objectSpread({}, state, {
          values: unstable_setIn(state.values, action.name, [].concat(array, [action.value]))
        });
      }

    case "remove":
      {
        var _array = unstable_getIn(state.values, action.name, []);

        delete _array[action.index];
        return _objectSpread({}, state, {
          values: unstable_setIn(state.values, action.name, _array)
        });
      }

    default:
      {
        throw new Error();
      }
  }
}

function unstable_useFormState(initialState) {
  if (initialState === void 0) {
    initialState = {};
  }

  var _unstable_useSealedSt = unstable_useSealedState(initialState),
      _unstable_useSealedSt2 = _unstable_useSealedSt.baseId,
      baseId = _unstable_useSealedSt2 === void 0 ? unstable_useId("form-") : _unstable_useSealedSt2,
      _unstable_useSealedSt3 = _unstable_useSealedSt.values,
      initialValues = _unstable_useSealedSt3 === void 0 ? {} : _unstable_useSealedSt3,
      _unstable_useSealedSt4 = _unstable_useSealedSt.validateOnBlur,
      validateOnBlur = _unstable_useSealedSt4 === void 0 ? true : _unstable_useSealedSt4,
      _unstable_useSealedSt5 = _unstable_useSealedSt.validateOnChange,
      validateOnChange = _unstable_useSealedSt5 === void 0 ? true : _unstable_useSealedSt5,
      _unstable_useSealedSt6 = _unstable_useSealedSt.resetOnSubmitSucceed,
      resetOnSubmitSucceed = _unstable_useSealedSt6 === void 0 ? false : _unstable_useSealedSt6,
      _unstable_useSealedSt7 = _unstable_useSealedSt.resetOnUnmount,
      resetOnUnmount = _unstable_useSealedSt7 === void 0 ? true : _unstable_useSealedSt7,
      onValidate = _unstable_useSealedSt.onValidate,
      onSubmit = _unstable_useSealedSt.onSubmit;

  var _React$useReducer = useReducer(reducer, {
    baseId: baseId,
    initialValues: initialValues,
    values: initialValues,
    touched: {},
    errors: {},
    messages: {},
    valid: true,
    validating: false,
    submitting: false,
    submitFailed: 0,
    submitSucceed: 0
  }),
      _React$useReducer$ = _React$useReducer[0],
      _ = _React$useReducer$.initialValues,
      state = _objectWithoutPropertiesLoose(_React$useReducer$, ["initialValues"]),
      dispatch = _React$useReducer[1];

  var validate = useCallback(function (vals) {
    if (vals === void 0) {
      vals = state.values;
    }

    return new Promise(function (resolve) {
      if (onValidate) {
        var response = onValidate(filterAllEmpty(vals));

        if (isPromise(response)) {
          dispatch({
            type: "startValidate"
          });
        }

        resolve(Promise.resolve(response).then(function (messages) {
          dispatch({
            type: "endValidate",
            messages: messages
          });
          return messages;
        }));
      } else {
        resolve(undefined);
      }
    })["catch"](function (errors) {
      dispatch({
        type: "endValidate",
        errors: errors
      });
      throw errors;
    });
  }, [state.values, onValidate]);
  useUpdateEffect(function () {
    if (validateOnChange) {
      validate()["catch"](function () {});
    }
  }, [validate, validateOnChange]);
  useEffect(function () {
    if (resetOnUnmount) {
      return function () {
        dispatch({
          type: "reset"
        });
      };
    }

    return undefined;
  }, [resetOnUnmount]);
  return _objectSpread({}, state, {
    values: state.values,
    validate: validate,
    reset: useCallback(function () {
      return dispatch({
        type: "reset"
      });
    }, []),
    submit: useCallback(function () {
      dispatch({
        type: "startSubmit"
      });
      return validate().then(function (validateMessages) {
        if (onSubmit) {
          return Promise.resolve(onSubmit(state.values)).then(function (submitMessages) {
            var messages = _objectSpread({}, validateMessages, submitMessages);

            dispatch({
              type: "endSubmit",
              messages: messages
            });

            if (resetOnSubmitSucceed) {
              dispatch({
                type: "reset"
              });
            }
          });
        }

        dispatch({
          type: "endSubmit",
          messages: validateMessages
        });
        return undefined;
      })["catch"](function (errors) {
        dispatch({
          type: "endSubmit",
          errors: errors
        });
      });
    }, [validate]),
    update: useCallback(function (name, value) {
      return dispatch({
        type: "update",
        name: name,
        value: value
      });
    }, []),
    blur: useCallback(function (name) {
      dispatch({
        type: "blur",
        name: name
      });

      if (validateOnBlur) {
        validate()["catch"](function () {});
      }
    }, [validate]),
    push: useCallback(function (name, value) {
      return dispatch({
        type: "push",
        name: name,
        value: value
      });
    }, []),
    remove: useCallback(function (name, index) {
      return dispatch({
        type: "remove",
        name: name,
        index: index
      });
    }, [])
  });
}
var keys = ["baseId", "values", "touched", "messages", "errors", "validating", "valid", "submitting", "submitSucceed", "submitFailed", "validate", "submit", "reset", "update", "blur", "push", "remove"];
unstable_useFormState.__keys = keys;

export { unstable_useFormState };
