import { useState, useEffect, useCallback, useContext, useMemo, createElement } from 'react';
import '../utils/useSealedState.js';
import '../utils/useId.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-94b779f5.js';
import { unstable_useCreateElement } from '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../system/useToken.js';
import '../system/useOptions.js';
import '../system/useProps.js';
import '../chunk-2f16e450.js';
import '../chunk-56719312.js';
import { unstable_createHook } from '../utils/createHook.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import { useBox } from '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../chunk-b94938c6.js';
import { a as useAllCallbacks } from '../chunk-a259d6a5.js';
import '../Tabbable/Tabbable.js';
import { a as warning } from '../chunk-79998200.js';
import '../Hidden/HiddenState.js';
import { a as usePipe } from '../chunk-3707f194.js';
import '../chunk-b4d83ecc.js';
import '../Dialog/DialogState.js';
import '../chunk-96f01ae3.js';
import '../Rover/RoverState.js';
import '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { a as MenuContext } from '../chunk-ee62c270.js';
import { useMenuState } from './MenuState.js';

function useShortcuts(_ref, timeout) {
  var stops = _ref.stops,
      move = _ref.move;

  if (timeout === void 0) {
    timeout = 500;
  }

  var _React$useState = useState(""),
      keys = _React$useState[0],
      setKeys = _React$useState[1];

  useEffect(function () {
    if (!keys) return undefined;
    var timeoutId = setTimeout(function () {
      return setKeys("");
    }, timeout);
    var stop = stops.find(function (s) {
      return Boolean(s.ref.current && s.ref.current.textContent && s.ref.current.textContent.toLowerCase().startsWith(keys));
    });

    if (stop) {
      move(stop.id);
    }

    return function () {
      return clearTimeout(timeoutId);
    };
  }, [keys, stops, move]);
  var onKeyDown = useCallback(function (event) {
    if (event.metaKey || event.altKey || event.shiftKey || event.ctrlKey) {
      return;
    }

    if (/^[a-z0-9_-]$/i.test(event.key)) {
      event.stopPropagation();
      event.preventDefault();
      setKeys("" + keys + event.key);
    }
  }, [keys, setKeys]);
  return onKeyDown;
}

var useStaticMenu = unstable_createHook({
  name: "StaticMenu",
  compose: useBox,
  useState: useMenuState,
  useProps: function useProps(options, _ref) {
    var htmlOnKeyDown = _ref.onKeyDown,
        htmlWrap = _ref.unstable_wrap,
        htmlProps = _objectWithoutPropertiesLoose(_ref, ["onKeyDown", "unstable_wrap"]);

    var parent = useContext(MenuContext);
    var providerValue = useMemo(function () {
      return {
        orientation: options.orientation,
        next: options.next,
        previous: options.previous,
        parent: parent
      };
    }, [options.orientation, options.next, options.previous, parent]);
    var onKeyDown = useShortcuts(options);
    var wrap = useCallback(function (children) {
      return createElement(MenuContext.Provider, {
        value: providerValue
      }, children);
    }, [providerValue]);
    return _objectSpread({
      role: options.orientation === "horizontal" ? "menubar" : "menu",
      "aria-orientation": options.orientation,
      onKeyDown: useAllCallbacks(onKeyDown, htmlOnKeyDown),
      unstable_wrap: usePipe(wrap, htmlWrap)
    }, htmlProps);
  }
});
var StaticMenu = unstable_createComponent({
  as: "div",
  useHook: useStaticMenu,
  useCreateElement: function useCreateElement(type, props, children) {
    process.env.NODE_ENV !== "production" ? warning(!props["aria-label"] && !props["aria-labelledby"] && props.role !== "menubar", "Menu", "You should provide either `aria-label` or `aria-labelledby` props.", "See https://reakit.io/docs/menu") : void 0;
    return unstable_useCreateElement(type, props, children);
  }
});

export { StaticMenu, useStaticMenu };
