'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useSealedState.js');
require('../utils/useId.js');
var __chunk_1 = require('../chunk-002fc018.js');
require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
require('../chunk-2138b837.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
require('../Box/Box.js');
require('../chunk-9263130a.js');
var __chunk_5 = require('../chunk-c7489df4.js');
var __chunk_6 = require('../chunk-471f8aea.js');
require('../Tabbable/Tabbable.js');
require('../Button/Button.js');
var __chunk_8 = require('../chunk-1299e245.js');
require('../Hidden/HiddenState.js');
require('../chunk-5a603fe6.js');
require('../Dialog/DialogState.js');
require('../Hidden/HiddenDisclosure.js');
require('../Dialog/DialogDisclosure.js');
var __chunk_16 = require('../chunk-498e747a.js');
require('../Rover/RoverState.js');
require('../Rover/Rover.js');
require('popper.js');
require('../Popover/PopoverState.js');
var __chunk_17 = require('../chunk-8a7a2ea8.js');
var MenuState = require('./MenuState.js');
var PopoverDisclosure = require('../Popover/PopoverDisclosure.js');

var noop = function noop() {};

var useMenuDisclosure = createHook.unstable_createHook({
  name: "MenuDisclosure",
  compose: PopoverDisclosure.usePopoverDisclosure,
  useState: MenuState.useMenuState,
  useProps: function useProps(options, _ref) {
    var htmlRef = _ref.ref,
        htmlOnClick = _ref.onClick,
        htmlOnKeyDown = _ref.onKeyDown,
        htmlOnFocus = _ref.onFocus,
        htmlOnMouseOver = _ref.onMouseOver,
        htmlProps = __chunk_1._objectWithoutPropertiesLoose(_ref, ["ref", "onClick", "onKeyDown", "onFocus", "onMouseOver"]);

    var parent = React.useContext(__chunk_17.MenuContext);
    var ref = React.useRef(null); // This avoids race condition between focus and click.
    // On some browsers, focus is triggered right before click.
    // So we use it to disable toggling.

    var _React$useState = React.useState(false),
        hasShownOnFocus = _React$useState[0],
        setHasShownOnFocus = _React$useState[1];

    var _options$placement$sp = options.placement.split("-"),
        dir = _options$placement$sp[0]; // Restores hasShownOnFocus


    React.useEffect(function () {
      if (hasShownOnFocus) {
        setTimeout(function () {
          return setHasShownOnFocus(false);
        }, 200);
      }
    }, [hasShownOnFocus]);
    var onKeyDown = React.useMemo(function () {
      return __chunk_16.createOnKeyDown({
        stopPropagation: function stopPropagation(event) {
          return event.key !== "Escape";
        },
        onKey: options.show,
        keyMap: function keyMap() {
          var first = function first() {
            return setTimeout(options.first);
          };

          return {
            Escape: options.hide,
            Enter: parent && first,
            " ": parent && first,
            ArrowUp: dir === "top" || dir === "bottom" ? options.last : false,
            ArrowRight: dir === "right" && first,
            ArrowDown: dir === "bottom" || dir === "top" ? first : false,
            ArrowLeft: dir === "left" && first
          };
        }
      });
    }, [options.show, options.hide, options.first, options.last]);
    var onFocus = React.useCallback(function () {
      if (parent && parent.orientation === "horizontal") {
        setHasShownOnFocus(true);
        options.show();
      }
    }, [parent && parent.orientation, setHasShownOnFocus, options.show]);
    var onMouseOver = React.useCallback(function () {
      if (!parent) return;

      if (!ref.current) {
        process.env.NODE_ENV !== "production" ? __chunk_8.warning(true, "MenuDisclosure", "Can't respond to mouse over on `MenuDisclosure` because `ref` wasn't passed to component.", "See https://reakit.io/docs/menu") : void 0;
        return;
      }

      var parentIsHorizontal = parent.orientation === "horizontal";

      if (!parentIsHorizontal) {
        setTimeout(function () {
          if (ref.current && ref.current.contains(document.activeElement)) {
            options.show();
            ref.current.focus();
          }
        }, 200);
      } else {
        var parentMenu = ref.current.closest("[role=menu],[role=menubar]");
        var subjacentOpenMenu = parentMenu && parentMenu.querySelector("[role=menu]:not([hidden])");

        if (subjacentOpenMenu) {
          ref.current.focus();
        }
      }
    }, [parent && parent.orientation, options.show]);
    var onClick = React.useCallback(function () {
      if (parent && (parent.orientation !== "horizontal" || hasShownOnFocus)) {
        options.show();
      } else {
        options.toggle();
      }
    }, [parent && parent.orientation, hasShownOnFocus, options.show, options.toggle]);
    return __chunk_1._objectSpread({
      ref: __chunk_5.mergeRefs(ref, htmlRef),
      "aria-haspopup": "menu",
      onClick: __chunk_6.useAllCallbacks(onClick, htmlOnClick),
      onKeyDown: __chunk_6.useAllCallbacks(onKeyDown, htmlOnKeyDown),
      onFocus: __chunk_6.useAllCallbacks(onFocus, htmlOnFocus),
      onMouseOver: __chunk_6.useAllCallbacks(onMouseOver, htmlOnMouseOver)
    }, htmlProps);
  },
  useCompose: function useCompose(options, htmlProps) {
    // Toggling is handled by MenuDisclosure
    return PopoverDisclosure.usePopoverDisclosure(__chunk_1._objectSpread({}, options, {
      toggle: noop
    }), htmlProps);
  }
});
var MenuDisclosure = createComponent.unstable_createComponent({
  as: "button",
  useHook: useMenuDisclosure
});

exports.MenuDisclosure = MenuDisclosure;
exports.useMenuDisclosure = useMenuDisclosure;
