'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useSealedState.js');
require('../utils/useId.js');
var __chunk_1 = require('../chunk-002fc018.js');
require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
require('../chunk-2138b837.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
require('../Box/Box.js');
require('../chunk-9263130a.js');
var __chunk_5 = require('../chunk-c7489df4.js');
var __chunk_6 = require('../chunk-471f8aea.js');
require('../Tabbable/Tabbable.js');
var __chunk_8 = require('../chunk-1299e245.js');
require('../Hidden/HiddenState.js');
require('../chunk-5a603fe6.js');
require('../Dialog/DialogState.js');
require('../chunk-498e747a.js');
require('../Rover/RoverState.js');
var Rover = require('../Rover/Rover.js');
require('popper.js');
require('../Popover/PopoverState.js');
require('../chunk-8a7a2ea8.js');
var MenuState = require('./MenuState.js');

// TODO: Find a better implementation
function isTouchDevice() {
  if (process.env.NODE_ENV === "test" || typeof window === "undefined") return false;
  return "ontouchstart" in window || navigator.maxTouchPoints > 0 || navigator.msMaxTouchPoints > 0;
}

var useMenuItem = createHook.unstable_createHook({
  name: "MenuItem",
  compose: Rover.useRover,
  useState: MenuState.useMenuState,
  useProps: function useProps(options, _ref) {
    var htmlRef = _ref.ref,
        htmlOnMouseOver = _ref.onMouseOver,
        htmlOnMouseOut = _ref.onMouseOut,
        htmlProps = __chunk_1._objectWithoutPropertiesLoose(_ref, ["ref", "onMouseOver", "onMouseOut"]);

    var ref = React.useRef(null);
    var onMouseOver = React.useCallback(function () {
      if (options.orientation !== "horizontal" && !isTouchDevice()) {
        if (!ref.current) {
          process.env.NODE_ENV !== "production" ? __chunk_8.warning(true, "MenuItem", "Can't respond to mouse over on `MenuItem` because `ref` wasn't passed to component.", "See https://reakit.io/docs/menu") : void 0;
          return;
        }

        ref.current.focus();
      }
    }, [options.orientation]);
    var onMouseOut = React.useCallback(function () {
      if (ref.current) {
        // Ignores disclosure
        if (!ref.current.hasAttribute("aria-controls") || ref.current.getAttribute("aria-expanded") !== "true") {
          ref.current.blur();
        }

        var menu = ref.current.closest("[role=menu],[role=menubar]");

        if (menu) {
          var nestedMenu = menu.querySelector("[role=menu]:not([hidden]),[role=menubar]:not([hidden])");

          if (!nestedMenu && !isTouchDevice()) {
            options.move(null);
            menu.focus();
          }
        }
      }
    }, [options.move]);
    return __chunk_1._objectSpread({
      ref: __chunk_5.mergeRefs(ref, htmlRef),
      role: "menuitem",
      onMouseOver: __chunk_6.useAllCallbacks(onMouseOver, htmlOnMouseOver),
      onMouseOut: __chunk_6.useAllCallbacks(onMouseOut, htmlOnMouseOut)
    }, htmlProps);
  }
});
var MenuItem = createComponent.unstable_createComponent({
  as: "button",
  useHook: useMenuItem
});

exports.MenuItem = MenuItem;
exports.useMenuItem = useMenuItem;
