'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useSealedState.js');
var useId = require('../utils/useId.js');
var __chunk_1 = require('../chunk-002fc018.js');
require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
require('../chunk-2138b837.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
require('../Box/Box.js');
require('../chunk-9263130a.js');
var __chunk_5 = require('../chunk-c7489df4.js');
var __chunk_6 = require('../chunk-471f8aea.js');
var Tabbable = require('../Tabbable/Tabbable.js');
var __chunk_8 = require('../chunk-1299e245.js');
var __chunk_10 = require('../chunk-5a603fe6.js');
var __chunk_16 = require('../chunk-498e747a.js');
var RoverState = require('./RoverState.js');

var useRover = createHook.unstable_createHook({
  name: "Rover",
  compose: Tabbable.useTabbable,
  useState: RoverState.useRoverState,
  keys: ["stopId"],
  useProps: function useProps(options, _ref) {
    var htmlRef = _ref.ref,
        htmlTabIndex = _ref.tabIndex,
        htmlOnFocus = _ref.onFocus,
        htmlOnKeyDown = _ref.onKeyDown,
        htmlProps = __chunk_1._objectWithoutPropertiesLoose(_ref, ["ref", "tabIndex", "onFocus", "onKeyDown"]);

    var ref = React.useRef(null);
    var id = useId.unstable_useId("rover-");
    var stopId = options.stopId || htmlProps.id || id;
    var trulyDisabled = options.disabled && !options.focusable;
    var noFocused = options.currentId == null;
    var focused = options.currentId === stopId;
    var isFirst = options.stops[0] && options.stops[0].id === stopId;
    var shouldTabIndex = focused || isFirst && noFocused;
    React.useEffect(function () {
      if (trulyDisabled) return undefined;
      options.register(stopId, ref);
      return function () {
        return options.unregister(stopId);
      };
    }, [stopId, trulyDisabled, options.register, options.unregister]);
    __chunk_10.useUpdateEffect(function () {
      if (!ref.current) {
        process.env.NODE_ENV !== "production" ? __chunk_8.warning(true, "Rover", "Can't focus rover component because `ref` wasn't passed to component.", "See https://reakit.io/docs/rover") : void 0;
        return;
      }

      if (focused) {
        ref.current.focus();
      }
    }, [focused]);
    var onFocus = React.useCallback(function () {
      return options.move(stopId);
    }, [options.move, stopId]);
    var onKeyDown = React.useMemo(function () {
      return __chunk_16.createOnKeyDown({
        onKeyDown: htmlOnKeyDown,
        keyMap: {
          ArrowUp: options.orientation !== "horizontal" && options.previous,
          ArrowRight: options.orientation !== "vertical" && options.next,
          ArrowDown: options.orientation !== "horizontal" && options.next,
          ArrowLeft: options.orientation !== "vertical" && options.previous,
          Home: options.first,
          End: options.last,
          PageUp: options.first,
          PageDown: options.last
        }
      });
    }, [htmlOnKeyDown, options.orientation, options.previous, options.next, options.first, options.last]);
    return __chunk_1._objectSpread({
      ref: __chunk_5.mergeRefs(ref, htmlRef),
      id: stopId,
      tabIndex: shouldTabIndex ? htmlTabIndex : -1,
      onKeyDown: onKeyDown,
      onFocus: __chunk_6.useAllCallbacks(onFocus, htmlOnFocus)
    }, htmlProps);
  }
});
var Rover = createComponent.unstable_createComponent({
  as: "button",
  useHook: useRover
});

exports.Rover = Rover;
exports.useRover = useRover;
